import numpy as np
from scipy import interpolate


def obs_data():

    alldata=np.loadtxt('gamma_TS10_4FGL_format.dat')
    timeg=alldata[:,0]
    fluxg=alldata[:,1]
    fluxgerr=alldata[:,2]

    alldata=np.loadtxt('OutburstR_residuals.dat',usecols=(0,1,2))
    time1=alldata[:,0]
    flux1=alldata[:,1]
    fluxerr1=alldata[:,2]
    alldata=np.loadtxt('BLLac_Pol_format.dat',usecols=(0,1,2,3,4,5,6))
    time2=alldata[:,0]
    flux2=alldata[:,1]
    fluxerr2=alldata[:,2]
    pold=alldata[:,3]
    polderr=alldata[:,4]
    pola=alldata[:,5]
    pola[pola>90]-=180
    pola[pola<-90]+=180
    polaerr=alldata[:,6]
    timeo=np.concatenate((time1,time2),axis=None)-2400000.5
    fluxotemp=np.concatenate((flux1,flux2),axis=None)
    fluxoerr=np.concatenate((fluxerr1,fluxerr2),axis=None)
    timep=time2-2400000.5

    fluxsin=20.0*np.sin((timeo-59075)/15.0*2.0*np.pi-0.5*np.pi)+70.0
    fluxo=fluxotemp-fluxsin

    return(timeg,fluxg,fluxgerr,timeo,fluxo,fluxoerr,timep,pold,polderr, \
      pola,polaerr)


def fit_data(fitargs,optargs,fitx):
    period,phase,bto_amp,bpo_amp,btu_amp,btu_ave=fitargs
    fluxgave,fluxoave,doppler,alpha,btu0=optargs
    timeg,timeo,timep,timef=fitx

    bto=bto_amp
    bpog=bpo_amp*np.sin(timeg/period*2.0*np.pi+phase)
    bpoo=bpo_amp*np.sin(timeo/period*2.0*np.pi+phase)
    bpop=bpo_amp*np.sin(timep/period*2.0*np.pi+phase)
    bpof=bpo_amp*np.sin(timef/period*2.0*np.pi+phase)
    btug=btu_amp*np.sin(timeg/period*2.0*np.pi+phase)
    btuo=btu_amp*np.sin(timeo/period*2.0*np.pi+phase)
    btup=btu_amp*np.sin(timep/period*2.0*np.pi+phase)
    btuf=btu_amp*np.sin(timef/period*2.0*np.pi+phase)

    factor=doppler**(alpha+3.0)
    emave=bto**2.0+0.5*bpo_amp**2.0+btu_ave
    emtemp=bto**2.0+bpog**2.0+btug**2.0+btu_ave-0.5*btu_amp**2.0
    emg=fluxgave*factor*emtemp/emave
    emtemp=bto**2.0+bpoo**2.0+btuo**2.0+btu_ave-0.5*btu_amp**2.0
    emo=fluxoave*factor*emtemp/emave
    emtemp=bto**2.0+bpop**2.0+btup**2.0+btu_ave-0.5*btu_amp**2.0
    pd=np.abs(bto**2.0-bpop**2.0)/(emtemp+btu0)*77
    pa=np.arctan2(bpop,bto)/np.pi*180.0
    fitmcmc=emg,emo,pd,pa

    emave=bto**2.0+0.5*bpo_amp**2.0+btu_ave
    tempnum=60
    tempx=np.linspace(timef[0],timef[-1],tempnum)
    tempy=np.random.normal(btu_ave,btu_amp,tempnum)

    tempy[tempy<0.0]=0.0
    btuf=interpolate.interp1d(tempx,tempy,kind='cubic')(timef)
    btuf[btuf<0.0]=0.0
    np.savetxt('turbulenceprofile.dat',btuf)
    emtemp=bto**2.0+bpof**2.0+btuf
#    emtemp=bto**2.0+bpof**2.0+btuf**2.0+btu_ave-0.5*btu_amp**2.0
    emgf=fluxgave*factor*emtemp/emave
    emof=fluxoave*factor*emtemp/emave
    pdf=np.abs(bto**2.0-bpof**2.0)/(emtemp+btu0)*77
    paf=np.arctan2(bpof,bto)/np.pi*180.0
    fit=emgf,emof,pdf,paf

    masktemp=np.logical_and(timep>=1,timep<=14)
    timetemp=timep[masktemp]
    btup=interpolate.interp1d(tempx,tempy,kind='cubic')(timetemp)
    btup[btup<0.0]=0.0
    emtemp=bto**2.0+bpop[masktemp]**2.0+btup
#    emtemp=bto**2.0+bpof**2.0+btuf**2.0+btu_ave-0.5*btu_amp**2.0
    emgp=fluxgave*factor*emtemp/emave
    emop=fluxoave*factor*emtemp/emave
    pdp=np.abs(bto**2.0-bpop[masktemp]**2.0)/(emtemp+btu0)*77
    pap=np.arctan2(bpop[masktemp],bto)/np.pi*180.0
    fitp=emgp,emop,pdp,pap

    masktemp=np.logical_and(timeg>=1,timeg<=14)
    timetemp=timeg[masktemp]
    btug=interpolate.interp1d(tempx,tempy,kind='cubic')(timetemp)
    btug[btug<0.0]=0.0
    emtemp=bto**2.0+bpog[masktemp]**2.0+btug
#    emtemp=bto**2.0+bpof**2.0+btuf**2.0+btu_ave-0.5*btu_amp**2.0
    emgg=fluxgave*factor*emtemp/emave
    emog=fluxoave*factor*emtemp/emave
    pdg=np.abs(bto**2.0-bpog[masktemp]**2.0)/(emtemp+btu0)*77
    pag=np.arctan2(bpog[masktemp],bto)/np.pi*180.0
    fitg=emgg,emog,pdg,pag

    masktemp=np.logical_and(timeo>=1,timeo<=14)
    timetemp=timeo[masktemp]
    btuo=interpolate.interp1d(tempx,tempy,kind='cubic')(timetemp)
    btuo[btuo<0.0]=0.0
    emtemp=bto**2.0+bpoo[masktemp]**2.0+btuo
#    emtemp=bto**2.0+bpof**2.0+btuf**2.0+btu_ave-0.5*btu_amp**2.0
    emgo=fluxgave*factor*emtemp/emave
    emoo=fluxoave*factor*emtemp/emave
    pdo=np.abs(bto**2.0-bpoo[masktemp]**2.0)/(emtemp+btu0)*77
    pao=np.arctan2(bpoo[masktemp],bto)/np.pi*180.0
    fito=emgo,emoo,pdo,pao

    return(fitmcmc,fit,fitp,fitg,fito)


def fitting():

    timeg,fluxg,fluxgerr,timeo,fluxo,fluxoerr,timep,pold,polderr, \
      pola,polaerr=obs_data()

    period=1.14
    phase=0.75*np.pi
    bto_amp=1.0
    bpo_amp=1.93
    btu_amp=2.8 #10.0
    btu_ave=14.6
    fluxgave=0.000242
    fluxoave=0.000109
    doppler=15.0
    alpha=1.2
    btu0=0.0
    timef=np.linspace(59075.0,59090.0,6000)
    timegtemp=timeg-59075.0
    timeotemp=timeo-59075.0
    timeptemp=timep-59075.0
    timeftemp=timef-59075.0
    fitargs=period,phase,bto_amp,bpo_amp,btu_amp,btu_ave
    optargs=fluxgave,fluxoave,doppler,alpha,btu0
    fitx=timegtemp,timeotemp,timeptemp,timeftemp
    _,fit,fitp,fitg,fito=fit_data(fitargs,optargs,fitx)
    emgf,emof,pdf,paf=fit
    emgp,emop,pdp,pap=fitp
    emgg,emog,pdg,pag=fitg
    emgo,emoo,pdo,pao=fito

    maskg=np.logical_and(timeg>=59076,timeg<=59089)
    masko=np.logical_and(timeo>=59076,timeo<=59089)
    maskp=np.logical_and(timep>=59076,timep<=59089)
    maskf=np.logical_and(timef>=59076,timef<=59089)

    result=np.column_stack((timef,emof,pdf,paf,emgf))
    np.savetxt('Run1.dat',result)


fitting()

